import yaml
import toml
# import xml.etree.ElementTree as ET
import json
from typing import Union



# def xml_dump(data: str) -> str:
#     # Turn a simple dict of key/value pairs into XML.
#     elem = ET.Element('result')
#     for key, val in data.items():
#         child = ET.SubElement(elem, key)
#         child.text = str(val)

#     return ET.tostring(elem, encoding='unicode')



def format_result(output_format: str, result: str) -> str:
	ks_dict = json.loads(result)
	if output_format:
		output_format = output_format.lower()
	
		if output_format == 'yaml':
			return yaml.dump(ks_dict, default_flow_style=False)
	
		elif output_format == 'toml':
			return toml.dumps(ks_dict)
	
		# elif output_format == 'xml':
		#	return xml_dump(ks_dict)
	
	return json.dumps(ks_dict, indent=5)


# Custom library imports:
from .config import *



class KsInvalidTokenError(Exception):
    def __init__(self):
        super().__init__('Invalid API token. After creating an account, generate a new one at https://kleenscan.com/profile.')



class KsApiError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(message)



class KsFileTooLargeError(Exception):
    def __init__(self):
        super().__init__(f'The provided file is too large for the kleenscan API (Max: {MAX_FILE_MB} MB).')



class KsFileEmptyError(Exception):
    def __init__(self):
        super().__init__(f'The provided file is empty, provide a file with data.')



class KsRemoteFileTooLargeError(Exception):
    def __init__(self):
        super().__init__(f'The remote file is too large for the kleenscan API (Max: {MAX_FILE_MB} MB).')



class KsGetFileInfoFailedError(Exception):
    def __init__(self, message: str):
        super().__init__(f'Failed to get file info, HTTP status code: {message}')



class KsNoFileHostedError(Exception):
    def __init__(self):
        super().__init__(f'No file hosted on provided URL/server. Please provide a URL/server which hosts a file, e.g. https://malicious.com/file.exe')



class KsFileDownloadFailedError(Exception):
    def __init__(self, message: str):
        super().__init__(f'Failed to download file, HTTP status code: {message}')



class KsDeadLinkError(Exception):
    def __init__(self, message: str):
        super().__init__(f'The URL/server hosting the file cannot be conneceted to: {message}')



class KsHttpError(Exception):
    def __init__(self, status_code: int):
        super().__init__(f'Kleenscan\'s server responded with a non 200 HTTP status code. This is likely due to rate-limits, if the status code is 429 a rate-limit was issued and too many requests were sent at once. Returned status code: {status_code}')


from setuptools import setup, find_packages

setup(
    name='kleenscan',
    version='0.5.8',
    packages=find_packages(),
    install_requires=[
        'toml',
        'pyyaml',
        'requests',
        'sty'
    ],
    entry_points={
        'console_scripts': [
            'kleenscan=kleenscan.cli:main',  # 'command-name=package.module:function'
        ],
    },
    author='Kleenscan',
    author_email='ksdev01@gmail.com',
    description='Kleenscan command line application and API wrapper library.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/ksdev01/kleenscan-cli',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)


import requests
import json
from typing import *


# Custom Library imports:
from .files import read_file
from .errors import *



class Ks_http:
	def __init__(self, x_auth_token: str):
		self.headers = {'X-Auth-Token': x_auth_token}



	@staticmethod
	def __handle_api_errors(response: requests.Response) -> None:
		if response.status_code != 200:
			raise KsHttpError(response.status_code)

		if "httpResponseCode" in response.text and '"httpResponseCode":200' not in response.text:
			raise KsApiError(response.text)



	def post_scan(self, url: str, data: dict, files: Optional[str]=None) -> dict:
		with requests.post(url,
			headers=self.headers,
			files=files,
			data=data
		) as response:
			self.__handle_api_errors(response)
			# print(response.text)
			return json.loads(response.text)



	def get_req(self, url: str) -> str:
		with requests.get(url,
			headers=self.headers,
		) as response:
			self.__handle_api_errors(response)
			# print(response.text)
			return response.text



	def post_req(self, url: str) -> str:
		with requests.post(url,
			headers=self.headers,
		) as response:
			self.__handle_api_errors(response)
			# print(response.text)
			return response.text



	def get_req_json(self, url: str) -> dict:
		with requests.get(url,
			headers=self.headers,
		) as response:
			self.__handle_api_errors(response)
			return json.loads(response.text)



	def get_req_json_noerr(self, url: str) -> dict:
		with requests.get(url,
			headers=self.headers,
		) as response:
			return json.loads(response.text)



	@staticmethod
	def download_file_memory(url: str) -> bytes:
		try:
			# Send a HEAD request to get the file size.
			with requests.head(url) as response:
				# Check if the request was successful.
				if response.status_code != 200:
					raise KsGetFileInfoFailedError(response.status_code)

				# Get the content type from the headers
				content_type = response.headers.get('Content-Type', '')

	    		# Check if the content type is HTML/text.
				if 'text/html' in content_type:
					raise KsNoFileHostedError
	    
				# Get the file size in MB from the Content-Length header.
				file_size_mb = int(response.headers.get('Content-Length', 0)) / 1024 / 1024

				# Check if the file size in MB exceeds the KS max size in MB.
				if file_size_mb >= MAX_FILE_MB:
					raise KsRemoteFileTooLargeError

				# Send a GET request to download the file.
				with requests.get(url, stream=True) as response:

					# Check if the request was successful.
					if response.status_code != 200:
						raise KsFileDownloadFailedError(response.status_code)

					# Return file content.
					return response.content

			return b''
		except (requests.ConnectionError, requests.Timeout, requests.RequestException) as e:
			raise KsDeadLinkError(f'{e} caused by {type(e)}')



